#pragma once

#include <Orange/Core/Types.h>
#include <string.h>

namespace orange
{
    template <typename T>
    struct Span
    {
        Span(T* data, size_t size) : data(data), size(size) {}

        template <size_t Count>
        Span(T (&array)[Count]) : data(array), size(Count) {}

        template <typename J>
        Span(J& x)
          : data(x.begin())
          , size(x.end() - x.begin()) {}

        explicit Span(T* data) : data(data), size(1) {}
        explicit Span(T& data) : data(&data), size(1) {}
        
              T& operator [] (size_t idx)       { return data[idx]; }
        const T& operator [] (size_t idx) const { return data[idx]; }

        template <typename J>
        bool operator == (const Span<J>& other) const
        {
            if (other.size != size)
                return false;

            for (size_t i = 0; i < size; i++)
            {
                if (data[i] != other.data[i])
                    return false;
            }

            return true;
        }

        bool IsEmpty() const { return size == 0; }

              T* begin()       { return &data[0]; }
        const T* begin() const { return &data[0]; }

              T* end()       { return &data[size]; }
        const T* end() const { return &data[size]; }

              T& front()       { return data[0]; }
        const T& front() const { return data[0]; }

              T& back()       { return data[size - 1]; }
        const T& back() const { return data[size - 1]; }

        T* data;
        size_t size;
    };

    using BufferView = Span<uint8_t>;

    struct StringView : public Span<const char>
    {
        using Span<const char>::Span;

        StringView(const char* str)
            : Span<const char>{str, strlen(str)} {}
    };

    struct Buffer : public BufferView
    {
        using BufferView::Span;

        ~Buffer() { delete[] data; }
    };

    struct TextBuffer : public Span<char>
    {
        using Span<char>::Span;

        ~TextBuffer() { delete[] data; }
    };
}
